program Debubble_moving
  ! Program for removing bubble from data
  ! Based on GEE's pef and helicon modules
  ! Calculates moving window containing reflections
  !   PEF is built based on those reflections
  !
  ! Inputs:
  !   1) history file of data
  !   2) nw: length of wavelet window
  !   3) t0:  zero-offset estimate (s)
  !   4) niter: number of iterations for pef
  !   5) nlag: length of filter
  !   6) ngap: length of gap
  !   7) offset: history file of offsets
  !   8) vel: water velocity
  !
  ! Outputs:
  !   1) history file of de-bubbled data
  !   2) window: optional output of data window
  !
  ! Bob Clapp
  ! March 2016


  !=================================
  ! standard declarations
  !=================================
  use sep
  use pef
  use helicon
  implicit none


  !=================================
  ! declare variables
  !=================================
  ! program variables
  integer                        :: ii,stat,it
  integer                        :: nlag,niter,ngap
  real                           :: tt,vel,t0

  ! input traces
  integer                        :: n1,n2,n123,n1o
  real                           :: o1,d1
  real,dimension(:,:),pointer    :: in2d,out2d,inw
  real, dimension(:),pointer     :: input,output
  real, dimension(:),allocatable :: offset

  ! wavelet variables
  integer                        :: nw,fw
  type(filter)                   :: aa


  !=================================
  ! open I/O
  !=================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1)
  call from_history('n2',n2,1)
  call from_history('o1',o1,0.)
  call from_history('d1',d1,1.)

  ! wavelet window parameters
  call from_param('nw',nw,n1)
  call from_param('t0',t0,0.)
  call from_param('vel',vel,1500.)
  call from_param("niter",niter,100)
  call from_param("nlag",nlag,25)
  call from_param("ngap",ngap,50)

  ! offset
  if (exist_file('offset')) then
    call from_aux('offset','n1',n1o)
  else
    call seperr('Need offest file')
  end if

  ! optional window output
  if (exist_file('window')) then
    call to_history('n1',n1,'window')
    call to_history('o1',o1,'window')
    call to_history('d1',d1,'window')
    call to_history('n2',n2,'window')
  end if


  !=================================
  ! initialization
  !=================================
  ! set variables
  n123=n1*n2

  ! allocate
  allocate(in2d(n1,n2),out2d(n1,n2))
  call allocatehelix(aa,nlag);
  allocate(aa%mis(n123))
  allocate(inw(nw,n2))
  allocate(offset(n2))


  !=================================
  ! kernel
  !=================================
  ! read in data and offsets
  call sep_read(in2d)
  call sep_read(offset,'offset')

  ! populate lags for filter
  ! essentially a gap deconvolution?
  do ii=1,nlag
    aa%lag(ii)=ii+ngap
  end do

  ! grab windowed portions
  do ii=1,n2
    tt = sqrt((t0*t0)+(offset(ii)*offset(ii)/vel/vel))
    it = 1.5+(tt-o1)/d1
    inw(:,ii) = in2d(it:it+nw,ii)
    out2d(:,ii) = 0.
    out2d(it:it+nw,ii) = in2d(it:it+nw,ii)
  end do
  if (exist_file('window')) then
    call srite('window',out2d,4*n1*n2)
  end if

  ! build pef
  allocate(input(size(inw)))
  input = reshape(inw,(/size(inw)/))
  call find_pef(input,aa,niter)
  deallocate(input)

  ! apply pet
  allocate(input(size(in2d)),output(size(in2d)))
  input = reshape(in2d,(/size(input)/))
  call helicon_init(aa)
  stat = helicon_lop(.false.,.false.,input,output)

  ! write out
  call sep_write(output)


  !=================================
  ! clean up
  !=================================
  call sep_close()
  call deallocatehelix(aa)
  deallocate(in2d,out2d,input,output,inw,offset)
  call exit(0)

end program
