program MatchG
  ! Program to shape geophone to look like hydrophone
  ! Uses fourier1d_mod
  !
  ! Inputs:
  !   1) history file of geophone
  !   2) fgeo: geophone impulse response
  !   3) fhyd: hydrophone impulse response
  !   4) filter: output for filter
  !
  ! Outputs:
  !   1) history file containing new geophone
  !
  ! Jason Chang
  ! February 2016

  !===================================
  ! standard declarations
  !===================================
  use sep
  use fourier1d_mod
  implicit none


  !===================================
  ! declare variables
  !===================================
  ! program variables
  integer                             :: ii,nn
  real                                :: eps
  real,allocatable,dimension(:)       :: temp

  ! input traces
  integer                             :: n1,n2,n3
  real                                :: o1,o2,o3,d1,d2,d3
  real                                :: epst
  real,allocatable,dimension(:)       :: data_t,gfilter_t

  ! impulse response traces
  integer                             :: n1g,n1h
  real                                :: o1g,o1h,d1g,d1h
  real                                :: epsir
  real,allocatable,dimension(:)       :: geo_t,hyd_t
  complex,allocatable,dimension(:)    :: geo_f,hyd_f

  ! fourier units
  integer                             :: nt,nw
  real                                :: rscale
  complex,allocatable,dimension(:)    :: data_f,gfilter_f
  

  !===================================
  ! open I/O
  !===================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  
  ! instrument response parameters
  if (exist_file('fgeo')) then
    call from_aux('fgeo','n1',n1g,1 )
    call from_aux('fgeo','o1',o1g,0.)
    call from_aux('fgeo','d1',d1g,1.)
  else
    call seperr('Need file with geophone impulse')
  end if

  ! hydrophone response parameters
  if (exist_file('fhyd')) then
    call from_aux('fhyd','n1',n1h,1 )
    call from_aux('fhyd','o1',o1h,0.)
    call from_aux('fhyd','d1',d1h,1.)
  else
    call seperr('Need file with hydrophone impulse')
  end if

  ! epsilon term
  call from_param('eps',eps,3.5e-3)



  !===================================
  ! initialize
  !===================================
  ! fourier
  nn = max(n1g,n1h,n1)
  call fft1d_init(nn)

  ! output
  !call to_history('n1',nn)
  !call to_history('n2',1 )
  !call to_history('n3',1 )
  !call to_history('o1',0.)
  !call to_history('o2',0.)
  !call to_history('o3',0.)
  !call to_history('d1',1.)
  !call to_history('d2',1.)
  !call to_history('d3',1.)

  ! optional filter output
  if (exist_file('filter')) then
    call to_history('n1',nn/2,'filter')
    call to_history('o1',0.,'filter')
    call to_history('d1',(1/d1g)/nn,'filter')
  end if

  ! allocate
  allocate(data_t(nn));    data_t=0.
  allocate(data_f(nn));    data_f=0.
  allocate(geo_t(nn));     geo_t=0.
  allocate(hyd_t(nn));     hyd_t=0.
  allocate(geo_f(nn));     geo_f=0.
  allocate(hyd_f(nn));     hyd_f=0.
  allocate(gfilter_t(nn)); gfilter_t=0.
  allocate(gfilter_f(nn)); gfilter_f=0.


  !===================================
  ! kernel
  !===================================
  ! read in responses
  call sreed('fgeo',geo_t(1:n1g),4*n1g)
  call sreed('fhyd',hyd_t(1:n1h),4*n1h)
  ! shift so that 0 sec is at the beginning
  geo_t = cshift(geo_t,shift=n1g/2)
  hyd_t = cshift(hyd_t,shift=n1h/2)
  geo_f = cmplx(geo_t,0.)
  hyd_f = cmplx(hyd_t,0.)

  ! fft the impulse responses
  call fft1d(.false.,1.,geo_f)
  call fft1d(.false.,1.,hyd_f)

  ! create filter by deconvolution
  ! note: epsilon only added to real component
  gfilter_f = hyd_f/(geo_f+eps)
  if (exist_file('filter')) then
    call srite('filter',cabs(gfilter_f(nn/2+1:nn)),4*(nn/2))
  end if

  ! loop over geophones
  do ii = 1,n2*n3
    ! read in geophones
    data_t = 0.
    call sreed('in',data_t(1:n1),4*n1)

    ! fft
    data_f = cmplx(data_t,0.)
    call fft1d(.false.,1.,data_f)

    ! apply the filter to geophone
    data_f = data_f*gfilter_f

    ! ifft
    call fft1d(.true.,1.,data_f)

    ! write out
    call srite('out',real(data_f(1:n1)),4*n1)

  end do


  !===================================
  ! clean up
  !===================================
  call fft1d_close()
  call sep_close()
  deallocate(data_t,data_f,geo_t,geo_f,hyd_t,hyd_f,gfilter_t,gfilter_f)
  call exit(0)

end program
