program MyPefDecon
  ! Program for removing bubble from data
  ! Based on GEE's pef and helicon modules
  ! Pef based on average of input traces
  !
  ! Inputs:
  !   1) history file of data
  !   2) nw: length of wavelet window
  !   3) fw: starting point of wavelet window
  !   4) niter: number of iterations for pef
  !   5) nlag: length of filter
  !   6) ngap: length of gap
  !   7) traces: input traces to be averaged
  !
  ! Outputs:
  !   1) history file of de-bubbled data
  !
  ! Bob Clapp
  ! February 2016


  !=================================
  ! standard declarations
  !=================================
  use sep
  use pef
  use helicon
  implicit none


  !=================================
  ! declare variables
  !=================================
  ! program variables
  integer                       :: ii,stat,iend
  integer                       :: nlag,niter,ngap

  ! input traces
  integer                       :: n1,n2,n123,n1a,n2a
  real,dimension(:,:),pointer   :: in2d,out2d
  real, dimension(:),pointer    :: input,output,inw,atrace

  ! wavelet variables
  integer                       :: nw,fw,nxl,nxr
  type(filter)                  :: aa


  !=================================
  ! open I/O
  !=================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1)
  call from_history('n2',n2,1)



  ! wavelet window parameters
  call from_param('nw',nw,n1)
  call from_param('fw',fw,0)
  call from_param('nxl',nxl,1)
  call from_param('nxr',nxr,n2)
  call from_param("niter",niter,100)
  call from_param("nlag",nlag,25)
  call from_param("ngap",ngap,50)


  !=================================
  ! initialization
  !=================================
  ! set variables
  n123=n1*n2

  ! allocate
  allocate(in2d(n1,n2),out2d(n1,n2),input(n1),output(n1))
  call allocatehelix(aa,nlag);
  allocate(aa%mis(n123))
  allocate(atrace(n1))


  !=================================
  ! kernel
  !=================================
  ! read in data
  call sep_read(in2d)


  ! populate lags for filter
  ! essentially a gap deconvolution?
  do ii=1,nlag
    aa%lag(ii)=ii+ngap
  end do

  ! calculate average trace for pef-building
  atrace = 0.
  do ii=nxl,nxr
    atrace = atrace+in2d(:,ii)
  end do
  atrace = atrace/(-nxl+nxr+1)
  iend = min(fw+nw,n1)
  allocate(inw(iend-fw))
  inw = atrace(fw+1:iend)

  ! build pef
  call find_pef(inw,aa,niter)
  
  ! loop over traces
  do ii=1,n2
    ! set input trace
    input=in2d(:,ii)
    call helicon_init(aa)

    ! convolve filter with data
    stat=helicon_lop(.false.,.false.,input,output)
    out2d(:,ii)=output
  end do

  ! write out
  call sep_write(out2d)


  !=================================
  ! clean up
  !=================================
  call sep_close()
  call deallocatehelix(aa)
  deallocate(in2d,out2d,input,output,inw,atrace)
  call exit(0)

end program
