program RemoveIR
  ! Program to deconvolve instrument response from recordings
  ! Uses fourier1d_mod
  !
  ! Inputs:
  !   1) history file of time series input
  !   2) fdecon: time series to deconvolve with
  !
  ! Outputs:
  !   1) history file containing deconvolved time series
  !
  ! Jason Chang
  ! October 2014

  !===================================
  ! standard declarations
  !===================================
  use sep
  use fourier1d_mod
  implicit none


  !===================================
  ! declare variables
  !===================================
  ! program variables
  integer                             :: ii,nn
  real                                :: eps
  real,allocatable,dimension(:)       :: temp

  ! input traces
  integer                             :: n1,n2,n3
  real                                :: o1,o2,o3,d1,d2,d3
  real                                :: epst
  real,allocatable,dimension(:)       :: data_t,trace_amp

  ! impulse response traces
  integer                             :: n1f,padd
  real                                :: o1f,d1f
  real                                :: epsir
  real,allocatable,dimension(:)       :: datair_t,ir_amp

  ! fourier units
  integer                             :: nt,nw
  real                                :: rscale
  real,allocatable,dimension(:)       :: fft_t
  complex,allocatable,dimension(:)    :: data_f,datair_f
  complex,allocatable,dimension(:)    :: fft_f
  integer*8                           :: fft_for,fft_inv
  

  !===================================
  ! open I/O
  !===================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  
  ! instrument response parameters
  if (exist_file('fdecon')) then
    call from_aux('fdecon','n1',n1f,1 )
    call from_aux('fdecon','o1',o1f,0.)
    call from_aux('fdecon','d1',d1f,1.)
  else
    call seperr('Need file containing trace to deconvolve with')
  end if

  ! epsilon term
  call from_param('eps',eps,3.5e-3)



  !===================================
  ! initialize
  !===================================
  ! fourier
  nn = max(n1f,n1)
  call fft1d_init(nn)

  ! allocate
  allocate(data_t(nn)); data_t=0.
  allocate(datair_t(nn)); datair_t=0.
  allocate(data_f (nn)); data_f=0.
  allocate(datair_f(nn)); datair_f=0.
  allocate(trace_amp(nn))
  allocate(ir_amp(nn))

  ! scale
  rscale = 1/sqrt(real(nn))


  !===================================
  ! kernel
  !===================================
  call sreed('fdecon',datair_t(1:n1f),4*n1f)
  ! shift so that 0 sec is at the beginning
  datair_t = cshift(datair_t,shift=n1f/2)
  datair_f = cmplx(datair_t,0.)

  ! fft the impulse response
  call fft1d(.false.,1.,datair_f)

  ! scale amplitude spectrum of impulse response (like normalization)
  !ir_amp = cabs(datair_f)
  !epsir = eps*(sum(ir_amp))/nn
  !datair_f = datair_f / (ir_amp+epsir)

  ! loop over receivers
  do ii = 1,n2*n3
    ! read in data
    data_t = 0.
    data_f = 0.
    call sreed('in',data_t(1:n1),4*n1)
    data_f = cmplx(data_t,0.)

    ! perform fft
    call fft1d(.false.,1.,data_f)

    ! scale amplitude spectrum of trace (like normalization)
    ! perform deconvolution
    data_f = (data_f*conjg(datair_f))/((datair_f*conjg(datair_f))+eps)

    ! perform ifft
    call fft1d(.true.,1.,data_f)
    data_t = rscale*real(data_f)

    ! write out data
    call srite('out',data_t(1:n1),4*n1)

  end do

  !===================================
  ! clean up
  !===================================
  call fft1d_close()
  call sep_close()
  deallocate(data_t,data_f,datair_t,datair_f,trace_amp,ir_amp)
  call exit(0)

end program
