program RemoveIR_resamp
  ! Program to deconvolve instrument response from recordings
  ! uses resampling of instrument response
  !
  ! Inputs:
  !   1) history file of time series input
  !   2) fdecon: time series to deconvolve with
  !
  ! Outputs:
  !   1) history file containing deconvolved time series
  !
  ! Jason Chang
  ! October 2014
  ! Update: January 2016
  !   - include resampling from Ettore's code


  !===================================
  ! standard declarations
  !===================================
  use sep
  use resample
  implicit none
  include "fftw3.f"


  !===================================
  ! declare variables
  !===================================
  ! program variables
  integer                             :: ii
  real                                :: eps
  real,allocatable,dimension(:)       :: temp

  ! input traces
  integer                             :: n1,n2,n3
  real                                :: o1,o2,o3,d1,d2,d3
  real                                :: epst
  real,allocatable,dimension(:)       :: data_t,trace_amp

  ! impulse response traces
  integer                             :: n1f,padd
  real                                :: o1f,d1f
  real                                :: epsir
  real,allocatable,dimension(:)       :: datair_t,ir_amp

  ! fourier units
  integer                             :: nt,nw
  real                                :: rscale
  real,allocatable,dimension(:)       :: fft_t
  complex,allocatable,dimension(:)    :: data_f,datair_f
  complex,allocatable,dimension(:)    :: fft_f
  integer*8                           :: fft_for,fft_inv
  

  !===================================
  ! open I/O
  !===================================
  call sep_init()

  ! history file parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)
  
  ! instrument response parameters
  if (exist_file('fdecon')) then
    call from_aux('fdecon','n1',n1f,1 )
    call from_aux('fdecon','o1',o1f,0.)
    call from_aux('fdecon','d1',d1f,1.)
  else
    call seperr('Need file containing trace to deconvolve with')
  end if

  ! epsilon term
  call from_param('eps',eps,3.5e-3)

  call sep_close()


  !===================================
  ! kernel
  !===================================

  ! calculate fourier units
  nt = n1
  nw = nt/2+1

  ! allocate
  allocate(data_t (nt)); data_t=0.
  allocate(datair_t(nt)); datair_t=0.
  allocate(data_f (nw)); data_f=0.
  allocate(datair_f(nw)); datair_f=0.
  allocate(fft_t  (nt))
  allocate(fft_f  (nw))
  allocate(temp (nt/2))
  allocate(trace_amp (nw))
  allocate(ir_amp (nw))

  ! prepare fft plans
  call sfftw_plan_dft_r2c_1d(fft_for,nt,fft_t,fft_f,FFTW_ESTIMATE)
  call sfftw_plan_dft_c2r_1d(fft_inv,nt,fft_f,fft_t,FFTW_ESTIMATE)
  rscale = 1./sqrt(real(nt))
  
  ! kernel
  call sreed('fdecon',datair_t,4*n1f)
  ! shift so that 0 sec is at the beginning
  datair_t = cshift(datair_t,shift=n1f/2)

  ! fft the impulse response
  fft_t = datair_t
  call sfftw_execute(fft_for)
  datair_f = fft_f
  datair_f = datair_f*rscale

  ! scale amplitude spectrum of impulse response (like normalization)
  ir_amp = cabs(datair_f)
  epsir = eps*(sum(ir_amp))/nt
  datair_f = datair_f / (ir_amp+epsir)


  ! loop over receivers
  do ii = 1,n2*n3
    ! read in data
    call sreed('in',data_t,4*n1)

    ! perform fft
    fft_t = data_t
    call sfftw_execute(fft_for)
    data_f = fft_f
    data_f = data_f*rscale

    ! scale amplitude spectrum of trace (like normalization)
    trace_amp = cabs(data_f)
    epst = eps*(sum(trace_amp))/nt
    data_f = data_f / (trace_amp+epst)

    ! perform deconvolution
    !data_f = rscale*(data_f/(datair_f+eps))
    data_f = rscale*data_f*conjg(datair_f)*(trace_amp/(ir_amp+epst))

    ! perform ifft
    fft_f = data_f
    call sfftw_execute(fft_inv)
    data_t = fft_t
    data_t = data_t*rscale

    ! write out data
    call srite('out',data_t,4*n1)

  end do

  !===================================
  ! clean up
  !===================================
  call sfftw_destroy_plan(fft_for,fft_inv)
  deallocate(data_t,data_f,datair_t,datair_f,fft_t,fft_f,temp,trace_amp,ir_amp)
  call exit(0)

end program
