program jcApplyFilter
  ! Program to apply filters to data
  ! Performed as a convolution
  !
  ! Inputs:
  !    1) history file of data
  !    2) filter: filter to apply to data
  !         - assumes filter does not need to be shifted
  !    3) shift: whether to fftshift filter
  !         - assumes that the response is centered
  !
  ! Outputs:
  !    1) history file of filtered data
  !
  ! Jason Chang
  ! February 2016


  !======================================
  ! standard declarations
  !======================================
  use sep
  use fourier1d_mod
  implicit none


  !======================================
  ! declare variables
  !======================================
  ! program variables
  integer                            :: ii
  integer                            :: nn
  logical                            :: shift

  ! input history file
  integer                            :: n1,n2,n3
  real                               :: o1,o2,o3,d1,d2,d3
  real,allocatable,dimension(:)      :: datat

  ! input filter file
  integer                            :: n1f
  real                               :: o1f,d1f
  real,allocatable,dimension(:)      :: filtt

  ! fourier
  complex,allocatable,dimension(:)   :: dataf,filtf
  


  !======================================
  ! open I/O
  !======================================
  call sep_init()

  ! input history file
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)

  ! filter history file
  if (exist_file('filter')) then
    call from_aux('filter','n1',n1f,1 )
    call from_aux('filter','o1',o1f,0.)
    call from_aux('filter','d1',d1f,1.)
  else
    call seperr('Need filter file filter=')
  end if

  ! shift
  call from_param('shift',shift,.false.)


  !======================================
  ! initialize
  !======================================
  ! number of samples
  nn = max(n1,n1f)

  ! allocate
  allocate(datat(nn)); datat=0.
  allocate(filtt(nn)); filtt=0.
  allocate(dataf(nn)); dataf=0.
  allocate(filtf(nn)); filtf=0.

  ! modules
  call fft1d_init(nn)


  !======================================
  ! kernel
  !======================================
  ! read in filter
  call sreed('filter',filtt(1:n1f),4*n1f)
  if (shift) then
    filtt = cshift(filtt,shift=n1f/2)
  end if
  filtf = cmplx(filtt,0.)
  call fft1d(.false.,1.,filtf)

  ! loop over traces
  do ii = 1,n2*n3
    ! read in data
    datat = 0.
    dataf = 0.
    call sreed('in',datat(1:n1),4*n1)
    dataf = cmplx(datat,0.)
    call fft1d(.false.,1.,dataf)

    ! convolve with filter
    dataf = dataf*filtf

    ! write out
    call fft1d(.true.,1.,dataf)
    datat = sqrt(real(nn))*real(dataf)
    call srite('out',datat(1:n1),4*n1)
  end do


  !======================================
  ! clean up
  !======================================
  call sep_close()
  call fft1d_close()
  deallocate(datat,dataf,filtt,filtf)
  call exit(0)

end program
