program jcGapDecon
  ! Program to calculate gap decon filter
  !    based on Digital Imaging ..., Robinson and Treitel
  ! Uses Levinson recursion to solve Toeplitz system of equations
  !    based on cwp code for solving recursion
  ! If number of traces is greater than one, then takes average of all traces
  !   - for filter length
  !
  ! Inputs:
  !   1) history file of trace
  !   2) eps: epsilon for adding noise
  !   3) topt: taper options for input traces
  !         0: hann-half
  !         1: hann
  !         2: hann-band
  !         3: boxcar
  !   4) t1,t2,t3,t4: times for tapers
  !   5) nlag: length of filter
  !   6) ngap: number of lags for gap
  !
  ! Outputs:
  !   1) history file of filtered traces
  !
  ! Jason Chang
  ! February 2016


  !======================================
  ! standard declarations
  !======================================
  use sep
  use fourier1d_mod
  use toeplitz_mod
  use td_convolution_mod
  implicit none


  !======================================
  ! declare variables
  !======================================
  ! program variables
  integer                            :: ii,jj
  integer                            :: counter,nlag,ngap
  real,allocatable,dimension(:)      :: rowin,colout,filter,dataout

  ! input history file
  integer                            :: n1,n2,n3
  real                               :: o1,o2,o3,d1,d2,d3
  real,allocatable,dimension(:)      :: datat
  real,allocatable,dimension(:,:)    :: dataauto

  ! decon variables
  real                               :: eps

  ! frequency domain
  complex,allocatable,dimension(:)   :: dataf


  !======================================
  ! open I/O
  !======================================
  call sep_init()

  ! input history file
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)

  ! decon parameters
  call from_param('eps',eps,1e-8)
  call from_param('nlag',nlag,20)
  call from_param('ngap',ngap,0)


  !======================================
  ! initialize
  !======================================
  ! allocate
  allocate(datat(n1));         datat=0.
  allocate(dataf(n1));         dataf=0.
  allocate(dataauto(n1,n2*n3)); dataauto=0.
  allocate(filter(nlag))      ; filter=0.
  allocate(rowin(nlag)) ; rowin=0.
  allocate(colout(nlag)); colout=0.

  ! modules
  call fft1d_init(n1)


  !======================================
  ! kernel
  !======================================
  ! compute autocorrelation of traces
  !   done for an approximation of source wavelet
  !   assumption is that reflectivity series is white
  !   therefore effect of reflections cancel out
  do ii=1,n2*n3
    ! read in traces
    call sreed('in',datat,4*n1)
    dataf = cmplx(datat,0.)

    ! perform fft
    call fft1d(.false.,1.,dataf)

    ! autocorrelation
    dataf = dataf*conjg(dataf)

    ! perform ifft
    call fft1d(.true.,1.,dataf)

    ! convert to real
    dataauto(:,ii) = real(dataf)
  end do

  ! write out autocorrelations (if interested)
  if (exist_file('auto')) call srite('auto',dataauto,4*n1*n2*n3)

  ! convolve data with filter
  call sseek('in',0.,0.)
  do ii=1,n2*n3
    ! filter program inputs
    filter = 0.
    rowin    = dataauto(1:nlag,ii)
    rowin(1) = rowin(1)*(1+eps)
    dataauto(:,ii)  = cshift(dataauto(:,ii),1)
    colout = dataauto(:,ii)
    !colout(1) = 1.

    ! actually calculate filter
    call toeplitz(rowin,colout,filter)

    ! time-domain convolution for each trace
    allocate(dataout(n1+nfilter(ii)-1)); dataout=0.

    call sreed('in',datat(1:n1),4*n1)
    call td_convolution(filter,datat,dataout)
    call srite('out',dataout(1:n1),4*n1)

  end do


  !======================================
  ! clean up
  !======================================
  ! output history file
  call to_history('n1',n1)
  call to_history('n2',n2)
  call to_history('n3',n3)
  call to_history('o1',o1)
  call to_history('o2',o2)
  call to_history('o3',o3)
  call to_history('d1',d1)
  call to_history('d2',d2)
  call to_history('d3',d3)

  ! optional autocorrelation  output
  if (exist_file('auto')) then
    call to_history('n1',n1,'auto')
    call to_history('n2',n2,'auto')
    call to_history('n3',n3,'auto')
    call to_history('o1',o1,'auto')
    call to_history('o2',o2,'auto')
    call to_history('o3',o3,'auto')
    call to_history('d1',d1,'auto')
    call to_history('d2',d2,'auto')
    call to_history('d3',d3,'auto')
  end if

  ! optional autocorrelation  output
  if (exist_file('filter')) then
    call to_history('n1',nfilter,'filter')
    call to_history('n2',1 ,'filter')
    call to_history('n3',1 ,'filter')
    call to_history('o1',o1,'filter')
    call to_history('o2',o2,'filter')
    call to_history('o3',o3,'filter')
    call to_history('d1',d1,'filter')
    call to_history('d2',d2,'filter')
    call to_history('d3',d3,'filter')
  end if

  call sep_close()
  call fft1d_close()
  deallocate(datat,dataf,dataauto)
  deallocate(rowin,colout,filter,dataout)
  call exit(0)

end program
