program jcSpikingDecon
  ! Program to calculate spiking decon filter
  ! Computes second zero-crossing of autocorrelation for operator length
  ! Uses Levinson recursion to solve Toeplitz system of equations
  ! If number of traces is greater than one, then takes average of all traces
  !   - for filter length
  !
  ! Inputs:
  !   1) history file of trace
  !   2) eps: epsilon for division
  !   3) topt: taper options
  !         0: hann-half
  !         1: hann
  !         2: hann-band
  !         3: boxcar
  !   4) t1,t2,t3,t4: times for tapers
  !
  ! Outputs:
  !   1) history file of filter
  !
  ! Jason Chang
  ! February 2016


  !======================================
  ! standard declarations
  !======================================
  use sep
  use fourier1d_mod
  use toeplitz_mod
  use td_convolution_mod
  implicit none


  !======================================
  ! declare variables
  !======================================
  ! program variables
  integer                            :: ii,jj,nfilter
  integer                            :: counter
  real,allocatable,dimension(:)      :: rowin,colout,filter,dataout

  ! input history file
  integer                            :: n1,n2,n3
  real                               :: o1,o2,o3,d1,d2,d3
  real,allocatable,dimension(:)      :: datat,datatot

  ! decon variables
  real                               :: eps

  ! frequency domain
  complex,allocatable,dimension(:)   :: dataf

  !======================================
  ! open I/O
  !======================================
  call sep_init()

  ! input history file
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)

  ! decon parameters
  call from_param('eps',eps,1e-8)


  !======================================
  ! initialize
  !======================================
  ! allocate
  allocate(datat(n1));   datat=0.
  allocate(dataf(n1));   dataf=0.
  allocate(datatot(n1)); datatot=0.

  ! modules
  call fft1d_init(n1)


  !======================================
  ! kernel
  !======================================
  do ii=1,n2*n3
    ! read in
    call sreed('in',datat,4*n1)
    dataf = cmplx(datat,0.)

    ! perform fft
    call fft1d(.false.,1.,dataf)

    ! autocorrelation
    dataf = dataf*conjg(dataf)

    ! perform ifft
    call fft1d(.true.,1.,dataf)

    ! convert to real
    datatot = datatot+real(dataf)
  end do

  ! write out
  datatot = datatot/(n2*n3)
  if (exist_file('auto')) call srite('auto',datatot,4*n1)

  ! pick second zero-crossing
  counter = 0
  do jj=2,n1
    if (sign(1.,datatot(jj)).ne.sign(1.,datatot(jj-1))) then
      counter = counter+1
    end if
    if (counter.eq.2) then
      nfilter = jj*3
      exit
    end if
  end do

  ! calculate filter
  call toeplitz_init()
  allocate(rowin(nfilter)) ; rowin=0.
  allocate(colout(nfilter)); colout=0.
  allocate(filter(nfilter)); filter=0.

  ! filter program inputs
  rowin    = datatot(1:nfilter)
  rowin(1) = rowin(1)*(1+eps)
  datatot  = cshift(datatot,1)
  colout(1) = 1.
  !colout   = datatot(1:nfilter)

  ! actually calculate filter
  call toeplitz(rowin,colout,filter)
  if (exist_file('filter')) call srite('filter',filter/filter(1),4*nfilter)
  ! check if filter is doing what it should be doing
  !write(0,*) sum(filter*rowin)
  !write(0,*) colout(1)
  !rowin = cshift(rowin,-1)
  !rowin(1) = rowin(3)
  !write(0,*) sum(filter*rowin)
  !write(0,*) colout(2)
  !rowin = cshift(rowin,-1)
  !rowin(1) = rowin(5)
  !write(0,*) sum(filter*rowin)
  !write(0,*) colout(3)

  ! apply filter to traces
  call sseek('in',0.,0.)
  allocate(dataout(n1+nfilter-1)); dataout=0.
  call td_convolution_init()

  do ii=1,n2*n3
    !deallocate(datat,filter)
    !allocate(datat(4)); datat=(/1.,3.,0.5,-1./)
    !allocate(dataout(6)); dataout=0.
    !allocate(filter(3)); filter = (/2.,-2.,1/)
    datat   = 0.
    dataout = 0.
    call sreed('in',datat(1:n1),4*n1)
    call td_convolution(filter,datat,dataout)
    call srite('out',dataout(1:n1),4*n1)
  end do


  !======================================
  ! clean up
  !======================================
  ! output history file
  call to_history('n1',n1)
  call to_history('n2',n2)
  call to_history('n3',n3)
  call to_history('o1',o1)
  call to_history('o2',o2)
  call to_history('o3',o3)
  call to_history('d1',d1)
  call to_history('d2',d2)
  call to_history('d3',d3)

  ! optional autocorrelation  output
  if (exist_file('auto')) then
    call to_history('n1',n1,'auto')
    call to_history('n2',1 ,'auto')
    call to_history('n3',1 ,'auto')
    call to_history('o1',o1,'auto')
    call to_history('o2',o2,'auto')
    call to_history('o3',o3,'auto')
    call to_history('d1',d1,'auto')
    call to_history('d2',d2,'auto')
    call to_history('d3',d3,'auto')
  end if

  ! optional autocorrelation  output
  if (exist_file('filter')) then
    call to_history('n1',nfilter,'filter')
    call to_history('n2',1 ,'filter')
    call to_history('n3',1 ,'filter')
    call to_history('o1',o1,'filter')
    call to_history('o2',o2,'filter')
    call to_history('o3',o3,'filter')
    call to_history('d1',d1,'filter')
    call to_history('d2',d2,'filter')
    call to_history('d3',d3,'filter')
  end if

  call sep_close()
  call fft1d_close()
  call toeplitz_close()
  call td_convolution_close()
  deallocate(datat,dataf,datatot,rowin,colout,filter,dataout)
  call exit(0)

end program
